@@ -9,7 +9,7 @@ use deno_runtime::deno_core::anyhow::bail;
 use deno_runtime::deno_core::error::AnyError;
 use deno_runtime::deno_core::{serde_v8, v8, Extension};
 
-use deno_core::op;
+use deno_core::{op, ModuleCode};
 use deno_runtime::deno_broadcast_channel::InMemoryBroadcastChannel;
 use deno_runtime::deno_core;
 use deno_runtime::deno_web::BlobStore;
@@ -17,13 +17,13 @@ use deno_runtime::permissions::{Permissions, PermissionsContainer};
 use deno_runtime::worker::MainWorker;
 use deno_runtime::worker::WorkerOptions;
 
+use deno_runtime::deno_fs::RealFs;
 use std::thread;
 use std::thread::JoinHandle;
 
 use crate::anyhow::anyhow;
 use futures::channel::{mpsc, mpsc::Sender, oneshot};
 use futures_util::{SinkExt, StreamExt};
-use resvg::FitTo;
 use usvg::{TreeParsing, TreeTextToPath};
 
 use crate::text::{op_text_width, FONT_DB, USVG_OPTIONS};
@@ -102,7 +102,7 @@ struct InnerVlConverter {
 impl InnerVlConverter {
     async fn init_vega(&mut self) -> Result<(), AnyError> {
         if !self.vega_initialized {
-            let import_code = format!(
+            let import_code = ModuleCode::from(format!(
                 r#"
 var vega;
 import('{vega_url}').then((imported) => {{
@@ -116,15 +116,15 @@ import('{vega_themes_url}').then((imported) => {{
 "#,
                 vega_url = vega_url(),
                 vega_themes_url = vega_themes_url(),
-            );
+            ));
 
-            self.worker.execute_script("<anon>", &import_code)?;
+            self.worker.execute_script("<anon>", import_code)?;
             self.worker.run_event_loop(false).await?;
 
             // Override text width measurement in vega-scenegraph
             for path in self.module_loader.import_map.keys() {
                 if path.ends_with("vega-scenegraph.js") {
-                    let script_code = format!(
+                    let script_code = ModuleCode::from(format!(
                         r#"
 import('{url}').then((sg) => {{
     sg.textMetrics.width = (item, text) => {{
@@ -146,8 +146,8 @@ import('{url}').then((sg) => {{
 }})
 "#,
                         url = url_for_path(path)
-                    );
-                    self.worker.execute_script("<anon>", &script_code)?;
+                    ));
+                    self.worker.execute_script("<anon>", script_code)?;
                     self.worker.run_event_loop(false).await?;
                 }
             }
@@ -162,7 +162,8 @@ function vegaToSvg(vgSpec) {
 }
 "#;
 
-            self.worker.execute_script("<anon>", function_str)?;
+            self.worker
+                .execute_script("<anon>", deno_core::FastString::Static(function_str))?;
             self.worker.run_event_loop(false).await?;
 
             self.vega_initialized = true;
@@ -174,7 +175,7 @@ function vegaToSvg(vgSpec) {
     async fn init_vl_version(&mut self, vl_version: &VlVersion) -> Result<(), AnyError> {
         if !self.initialized_vl_versions.contains(vl_version) {
             // Create and evaluate import string
-            let import_code = format!(
+            let import_code = ModuleCode::from(format!(
                 r#"
 var {ver_name};
 import('{vl_url}').then((imported) => {{
@@ -183,14 +184,14 @@ import('{vl_url}').then((imported) => {{
 "#,
                 ver_name = format!("{:?}", vl_version),
                 vl_url = vl_version.to_url()
-            );
+            ));
 
-            self.worker.execute_script("<anon>", &import_code)?;
+            self.worker.execute_script("<anon>", import_code)?;
 
             self.worker.run_event_loop(false).await?;
 
             // Create and initialize function string
-            let function_code = format!(
+            let function_code = ModuleCode::from(format!(
                 r#"
 function compileVegaLite_{ver_name}(vlSpec, config, theme) {{
     let options = {{}};
@@ -224,9 +225,9 @@ function vegaLiteToSvg_{ver_name}(vlSpec, config, theme) {{
 }}
 "#,
                 ver_name = format!("{:?}", vl_version),
-            );
+            ));
 
-            self.worker.execute_script("<anon>", &function_code)?;
+            self.worker.execute_script("<anon>", function_code)?;
 
             self.worker.run_event_loop(false).await?;
 
@@ -257,10 +258,10 @@ function vegaLiteToSvg_{ver_name}(vlSpec, config, theme) {{
         let options = WorkerOptions {
             bootstrap: Default::default(),
             extensions: vec![ext],
-            extensions_with_js: vec![],
             startup_snapshot: None,
+            create_params: None,
             unsafely_ignore_certificate_errors: None,
-            root_cert_store: None,
+            root_cert_store_provider: None,
             seed: None,
             source_map_getter: None,
             format_js_error_fn: None,
@@ -280,10 +281,11 @@ function vegaLiteToSvg_{ver_name}(vlSpec, config, theme) {{
             compiled_wasm_module_store: None,
             stdio: Default::default(),
             should_wait_for_inspector_session: false,
+            fs: Arc::new(RealFs),
         };
 
-        let js_path = Path::new(env!("CARGO_MANIFEST_DIR")).join("vl-convert-rs.js");
-        let main_module = deno_core::resolve_path(&js_path.to_string_lossy())?;
+        let main_module =
+            deno_core::resolve_path("vl-convert-rs.js", Path::new(env!("CARGO_MANIFEST_DIR")))?;
         let permissions = PermissionsContainer::new(Permissions::allow_all());
 
         let mut worker =
@@ -305,7 +307,8 @@ function vegaLiteToSvg_{ver_name}(vlSpec, config, theme) {{
         &mut self,
         script: &str,
     ) -> Result<serde_json::Value, AnyError> {
-        let res = self.worker.js_runtime.execute_script("<anon>", script)?;
+        let code = ModuleCode::from(script.to_string());
+        let res = self.worker.js_runtime.execute_script("<anon>", code)?;
 
         self.worker.run_event_loop(false).await?;
 
@@ -324,7 +327,8 @@ function vegaLiteToSvg_{ver_name}(vlSpec, config, theme) {{
     }
 
     async fn execute_script_to_string(&mut self, script: &str) -> Result<String, AnyError> {
-        let res = self.worker.js_runtime.execute_script("<anon>", script)?;
+        let code = ModuleCode::from(script.to_string());
+        let res = self.worker.js_runtime.execute_script("<anon>", code)?;
 
         self.worker.run_event_loop(false).await?;
 
@@ -397,7 +401,7 @@ compileVegaLite_{ver_name:?}(
             Some(s) => format!("'{}'", s),
         };
 
-        let code = format!(
+        let code = ModuleCode::from(format!(
             r#"
 var svg;
 vegaLiteToSvg_{ver_name:?}(
@@ -412,8 +416,8 @@ vegaLiteToSvg_{ver_name:?}(
             spec_arg_id = spec_arg_id,
             config_arg_id = config_arg_id,
             theme_arg = theme_arg,
-        );
-        self.worker.execute_script("<anon>", &code)?;
+        ));
+        self.worker.execute_script("<anon>", code)?;
         self.worker.run_event_loop(false).await?;
 
         let value = self.execute_script_to_string("svg").await?;
@@ -424,7 +428,7 @@ vegaLiteToSvg_{ver_name:?}(
         self.init_vega().await?;
 
         let arg_id = set_json_arg(vg_spec.clone())?;
-        let code = format!(
+        let code = ModuleCode::from(format!(
             r#"
 var svg;
 vegaToSvg(
@@ -434,8 +438,8 @@ vegaToSvg(
 }})
 "#,
             arg_id = arg_id
-        );
-        self.worker.execute_script("<anon>", &code)?;
+        ));
+        self.worker.execute_script("<anon>", code)?;
         self.worker.run_event_loop(false).await?;
 
         let value = self.execute_script_to_string("svg").await?;
@@ -443,7 +447,10 @@ vegaToSvg(
     }
 
     pub async fn get_local_tz(&mut self) -> Result<Option<String>, AnyError> {
-        let code = "var localTz = Intl.DateTimeFormat().resolvedOptions().timeZone ?? 'undefined';";
+        let code = ModuleCode::from(
+            "var localTz = Intl.DateTimeFormat().resolvedOptions().timeZone ?? 'undefined';"
+                .to_string(),
+        );
         self.worker.execute_script("<anon>", code)?;
         self.worker.run_event_loop(false).await?;
 
@@ -458,11 +465,14 @@ vegaToSvg(
     pub async fn get_themes(&mut self) -> Result<serde_json::Value, AnyError> {
         self.init_vega().await?;
 
-        let code = r#"
+        let code = ModuleCode::from(
+            r#"
 var themes = Object.assign({}, vegaThemes);
 delete themes.version
 delete themes.default
-"#;
+"#
+            .to_string(),
+        );
         self.worker.execute_script("<anon>", code)?;
         self.worker.run_event_loop(false).await?;
 
@@ -705,23 +715,19 @@ impl VlConverter {
                 bail!("Failed to parse SVG string: {}", err.to_string())
             }
         };
-
         rtree.convert_text(&font_database);
 
-        let pixmap_size = rtree.size.to_screen_size();
+        let rtree = resvg::Tree::from_usvg(&rtree);
+
         let mut pixmap = tiny_skia::Pixmap::new(
-            (pixmap_size.width() as f32 * scale) as u32,
-            (pixmap_size.height() as f32 * scale) as u32,
-        )
-        .unwrap();
-        resvg::render(
-            &rtree,
-            FitTo::Zoom(scale),
-            tiny_skia::Transform::default(),
-            pixmap.as_mut(),
+            (rtree.size.width() * scale) as u32,
+            (rtree.size.height() * scale) as u32,
         )
         .unwrap();
 
+        let transform = tiny_skia::Transform::from_scale(scale, scale);
+        resvg::Tree::render(&rtree, transform, &mut pixmap.as_mut());
+
         match pixmap.encode_png() {
             Ok(png_data) => Ok(png_data),
             Err(err) => {
