@@ -1,9 +1,12 @@
+use dssim::{Dssim, DssimImage};
 use rstest::rstest;
 use std::fs;
+use std::io::Write;
 use std::path::{Path, PathBuf};
 use vl_convert_rs::text::register_font_directory;
 use vl_convert_rs::{VlConverter, VlVersion};
 
+use serde_json::Value;
 use std::sync::Once;
 use vl_convert_rs::converter::VlOpts;
 
@@ -48,6 +51,42 @@ fn load_expected_vg_spec(name: &str, vl_version: VlVersion) -> Option<serde_json
     }
 }
 
+fn write_failed_vg(name: &str, vl_version: VlVersion, vg_spec: Option<Value>) {
+    if let Some(vg_spec) = vg_spec {
+        let root_path = Path::new(env!("CARGO_MANIFEST_DIR"));
+        let failed_dir = root_path
+            .join("tests")
+            .join("vl-specs")
+            .join("failed")
+            .join(format!("{:?}", vl_version));
+
+        fs::create_dir_all(failed_dir.clone()).unwrap();
+
+        // Write standard
+        let file_path = failed_dir.join(format!("{}.vg.json", name));
+        let mut file = fs::File::create(file_path.clone()).unwrap();
+        file.write_all(serde_json::to_string(&vg_spec).unwrap().as_bytes())
+            .unwrap();
+
+        // Write pretty
+        let file_path = failed_dir.join(format!("{}.vg.pretty.json", name));
+        let mut file = fs::File::create(file_path.clone()).unwrap();
+        file.write_all(serde_json::to_string_pretty(&vg_spec).unwrap().as_bytes())
+            .unwrap();
+    }
+}
+
+fn check_vg(name: &str, vl_version: VlVersion, vg_spec: Option<Value>) {
+    let expected = load_expected_vg_spec(name, vl_version);
+    if vg_spec != expected {
+        let path = write_failed_vg(name, vl_version, vg_spec);
+        panic!(
+            "Images don't match for {}.svg. Failed image written to {:?}",
+            name, path
+        )
+    }
+}
+
 fn make_expected_svg_path(name: &str, vl_version: VlVersion, theme: Option<&str>) -> PathBuf {
     let root_path = Path::new(env!("CARGO_MANIFEST_DIR"));
     root_path
@@ -67,6 +106,38 @@ fn load_expected_svg(name: &str, vl_version: VlVersion, theme: Option<&str>) ->
     fs::read_to_string(&spec_path).unwrap_or_else(|_| panic!("Failed to read {:?}", spec_path))
 }
 
+fn write_failed_svg(name: &str, vl_version: VlVersion, theme: Option<&str>, img: &str) -> PathBuf {
+    let root_path = Path::new(env!("CARGO_MANIFEST_DIR"));
+    let failed_dir = root_path
+        .join("tests")
+        .join("vl-specs")
+        .join("failed")
+        .join(format!("{:?}", vl_version));
+
+    fs::create_dir_all(failed_dir.clone()).unwrap();
+
+    let file_path = failed_dir.join(if let Some(theme) = theme {
+        format!("{}-{}.svg", name, theme)
+    } else {
+        format!("{}.svg", name)
+    });
+
+    let mut file = fs::File::create(file_path.clone()).unwrap();
+    file.write_all(img.as_bytes()).unwrap();
+    return file_path;
+}
+
+fn check_svg(name: &str, vl_version: VlVersion, theme: Option<&str>, img: &str) {
+    let expected = load_expected_svg(name, vl_version, theme);
+    if img != expected {
+        let path = write_failed_svg(name, vl_version, None, img);
+        panic!(
+            "Images don't match for {}.svg. Failed image written to {:?}",
+            name, path
+        )
+    }
+}
+
 fn make_expected_png_path(name: &str, vl_version: VlVersion, theme: Option<&str>) -> PathBuf {
     let root_path = Path::new(env!("CARGO_MANIFEST_DIR"));
     root_path
@@ -81,9 +152,57 @@ fn make_expected_png_path(name: &str, vl_version: VlVersion, theme: Option<&str>
         })
 }
 
-fn load_expected_png(name: &str, vl_version: VlVersion, theme: Option<&str>) -> Vec<u8> {
+fn load_expected_png_dssim(
+    name: &str,
+    vl_version: VlVersion,
+    theme: Option<&str>,
+) -> DssimImage<f32> {
     let spec_path = make_expected_png_path(name, vl_version, theme);
-    fs::read(&spec_path).unwrap_or_else(|_| panic!("Failed to read {:?}", spec_path))
+    dssim::load_image(&Dssim::new(), spec_path).unwrap()
+}
+
+fn to_dssim(img: &[u8]) -> DssimImage<f32> {
+    let mut tmpfile = tempfile::NamedTempFile::new().unwrap();
+    tmpfile.write_all(img).unwrap();
+    dssim::load_image(&Dssim::new(), tmpfile.path()).unwrap()
+}
+
+fn write_failed_png(name: &str, vl_version: VlVersion, theme: Option<&str>, img: &[u8]) -> PathBuf {
+    let root_path = Path::new(env!("CARGO_MANIFEST_DIR"));
+    let failed_dir = root_path
+        .join("tests")
+        .join("vl-specs")
+        .join("failed")
+        .join(format!("{:?}", vl_version));
+
+    fs::create_dir_all(failed_dir.clone()).unwrap();
+
+    let file_path = failed_dir.join(if let Some(theme) = theme {
+        format!("{}-{}.png", name, theme)
+    } else {
+        format!("{}.png", name)
+    });
+
+    let mut file = fs::File::create(file_path.clone()).unwrap();
+    file.write_all(img).unwrap();
+    return file_path;
+}
+
+fn check_png(name: &str, vl_version: VlVersion, theme: Option<&str>, img: &[u8]) {
+    let expected_dssim = load_expected_png_dssim(name, vl_version, theme);
+    let img_dssim = to_dssim(img);
+
+    let attr = Dssim::new();
+    let (diff, _) = attr.compare(&expected_dssim, img_dssim);
+
+    if diff > 0.0001 {
+        println!("DSSIM diff {diff}");
+        let path = write_failed_png(name, vl_version, None, img);
+        panic!(
+            "Images don't match for {}.png. Failed image written to {:?}",
+            name, path
+        )
+    }
 }
 
 #[rustfmt::skip]
@@ -104,6 +223,8 @@ mod test_vegalite_to_vega {
             VlVersion::v5_7,
             VlVersion::v5_8,
             VlVersion::v5_9,
+            VlVersion::v5_10,
+            VlVersion::v5_11,
         )]
         vl_version: VlVersion,
 
@@ -121,20 +242,9 @@ mod test_vegalite_to_vega {
         let vg_result = block_on(
             converter.vegalite_to_vega(vl_spec, VlOpts{vl_version, ..Default::default()}
             )
-        );
+        ).ok();
 
-        match load_expected_vg_spec(name, vl_version) {
-            Some(expected_vg_spec) => {
-                // Conversion is expected to succeed and match this
-                let vg_result = vg_result.expect("Vega-Lite to Vega conversion failed");
-                assert_eq!(vg_result, expected_vg_spec)
-            }
-            None => {
-                println!("{:?}", vg_result);
-                // Conversion is expected to fail
-                assert!(vg_result.is_err())
-            }
-        }
+        check_vg(name, vl_version, vg_result);
     }
 
     #[test]
@@ -155,7 +265,8 @@ mod test_svg {
             "stacked_bar_h",
             "bar_chart_trellis_compact",
             "line_with_log_scale",
-            "numeric_font_weight"
+            "numeric_font_weight",
+            "float_font_size",
         )]
         name: &str,
     ) {
@@ -166,9 +277,6 @@ mod test_svg {
         // Load example Vega-Lite spec
         let vl_spec = load_vl_spec(name);
 
-        // Load expected SVG image
-        let expected_svg = load_expected_svg(name, vl_version, None);
-
         // Create Vega-Lite Converter and perform conversion
         let mut converter = VlConverter::new();
 
@@ -177,15 +285,11 @@ mod test_svg {
             block_on(converter.vegalite_to_vega(vl_spec.clone(), VlOpts{vl_version, ..Default::default()})).unwrap();
 
         let svg = block_on(converter.vega_to_svg(vg_spec)).unwrap();
-        assert_eq!(svg, expected_svg);
+        check_svg(name, vl_version, None, &svg);
 
         // Convert directly to svg
         let svg = block_on(converter.vegalite_to_svg(vl_spec, VlOpts{vl_version, ..Default::default()})).unwrap();
-        assert_eq!(svg, expected_svg);
-
-        // // Write out reference image
-        // let svg_path = make_expected_svg_path(name, vl_version, None);
-        // std::fs::write(svg_path, svg).unwrap();
+        check_svg(name, vl_version, None, &svg);
     }
 
     #[test]
@@ -206,6 +310,7 @@ mod test_png_no_theme {
         case("line_with_log_scale", 2.0),
         case("remote_images", 1.0),
         case("maptile_background", 1.0),
+        case("float_font_size", 1.0),
     )]
     fn test(
         name: &str,
@@ -218,9 +323,6 @@ mod test_png_no_theme {
         // Load example Vega-Lite spec
         let vl_spec = load_vl_spec(name);
 
-        // Load expected PNG image
-        let expected_png_data = load_expected_png(name, vl_version, None);
-
         // Create Vega-Lite Converter and perform conversion
         let mut converter = VlConverter::new();
 
@@ -230,17 +332,13 @@ mod test_png_no_theme {
         ).unwrap();
 
         let png_data = block_on(converter.vega_to_png(vg_spec, Some(scale))).unwrap();
-        assert_eq!(png_data, expected_png_data);
+        check_png(name, vl_version, None, png_data.as_slice());
 
         // Convert directly to png
         let png_data = block_on(
             converter.vegalite_to_png(vl_spec, VlOpts{vl_version, ..Default::default()}, Some(scale))
         ).unwrap();
-        assert_eq!(png_data, expected_png_data);
-
-        // // Write out reference image
-        // let png_path = make_expected_png_path(name, vl_version, None);
-        // std::fs::write(png_path, png_data).unwrap();
+        check_png(name, vl_version, None, png_data.as_slice());
     }
 
     #[test]
@@ -273,9 +371,6 @@ mod test_png_theme_config {
         // Load example Vega-Lite spec
         let vl_spec = load_vl_spec(name);
 
-        // Load expected PNG image
-        let expected_png_data = load_expected_png(name, vl_version, Some(theme));
-
         // Create Vega-Lite Converter and perform conversion
         let mut converter = VlConverter::new();
 
@@ -290,7 +385,7 @@ mod test_png_theme_config {
                 }, Some(scale)
             )
         ).unwrap();
-        assert_eq!(png_data, expected_png_data);
+        check_png(name, vl_version, Some(theme), png_data.as_slice());
 
         // Patch spec to put theme in `vl_spec.usermeta.embedOptions.theme` and don't pass theme
         // argument
@@ -310,11 +405,7 @@ mod test_png_theme_config {
                 }, Some(scale)
             )
         ).unwrap();
-        assert_eq!(png_data, expected_png_data);
-
-        // // Write out reference image
-        // let png_path = make_expected_png_path(name, vl_version, Some(theme));
-        // std::fs::write(png_path, png_data).unwrap();
+        check_png(name, vl_version, Some(theme), png_data.as_slice());
     }
 
     #[test]
@@ -332,9 +423,6 @@ async fn test_font_with_quotes() {
     // Create Vega-Lite Converter and perform conversion
     let mut converter = VlConverter::new();
 
-    // Load expected PNG image
-    let expected_png_data = load_expected_png(name, vl_version, None);
-
     let png_data = converter
         .vegalite_to_png(
             vl_spec,
@@ -347,9 +435,5 @@ async fn test_font_with_quotes() {
         .await
         .unwrap();
 
-    assert_eq!(png_data, expected_png_data);
-
-    // // Write out reference image
-    // let png_path = make_expected_png_path(name, vl_version, None);
-    // std::fs::write(png_path, png_data).unwrap();
+    check_png(name, vl_version, None, png_data.as_slice());
 }
